local addonName, zrMM = ...

function zrMM:CreateButtonFrame()

    -- Button frame
    Minimap.buttonFrame = CreateFrame("frame", "zrButtonFrame", Minimap)
    Minimap.buttonFrame:RegisterEvent("PLAYER_ENTERING_WORLD")
    Minimap.buttonFrame:RegisterEvent("PLAYER_XP_UPDATE")
    Minimap.buttonFrame:RegisterEvent("PLAYER_LEVEL_UP")
    Minimap.buttonFrame:RegisterEvent("UPDATE_FACTION")
    
    
    

    local zrConfigButton = CreateFrame("button","zrMM_configButton", Minimap)
    zrConfigButton.text = zrConfigButton:CreateFontString(nil,"OVERLAY")
    zrConfigButton.text:SetFontObject("zrMM_FONT_SMALL")
    zrConfigButton.text:SetTextColor(.4,.6,1)
    zrConfigButton.text:SetText("zr")
    zrConfigButton.text:SetJustifyH("CENTER")
    zrConfigButton.text:SetPoint("CENTER", zrConfigButton, "CENTER", -1, -1)
    zrConfigButton:RegisterForClicks("AnyUp")
    zrConfigButton:SetScript("OnEnter", function(self) 
        self.text:SetTextColor(.6,.8,1) 
        GameTooltip:SetOwner(self, "ANCHOR_TOPLEFT", 0, 6)
        GameTooltip:AddLine("zrMM Config")
        GameTooltip:Show()
    end)
    zrConfigButton:SetScript("OnLeave", function(self) 
        self.text:SetTextColor(.4,.6,1)
        GameTooltip:Hide()
    end)
    zrConfigButton:SetScript("OnClick", function(self, button)
        if (button == "LeftButton") then
            zrSettings:Show()
        end
    end)


    -- Find and move buttons
	zrButtonFrame.frameTable = {}
    local ignoreFrames = {}
    local hideTextures = {}
    local manualTarget = {}
    local hideButtons = {}
    local numChildren = 0
	
	
    manualTarget['CodexBrowserIcon'] = true                             
    manualTarget['MiniMapTracking'] = true
    manualTarget['MiniMapTrackingFrame'] = true
    manualTarget['MiniMapMailFrame'] = true
    manualTarget['COHCMinimapButton'] = true
    manualTarget['ZygorGuidesViewerMapIcon'] = true
    manualTarget['MiniMapBattlefieldFrame'] = true
    manualTarget['PeggledMinimapIcon'] = true

    ignoreFrames['zrButtonFrame'] = true
    ignoreFrames['MinimapBackdrop'] = true
    ignoreFrames['GameTimeFrame'] = true
    ignoreFrames['MinimapVoiceChatFrame'] = true
    ignoreFrames['TimeManagerClockButton'] = true

    hideTextures['Interface\\Minimap\\MiniMap-TrackingBorder'] = true
    hideTextures['Interface\\Minimap\\UI-Minimap-ZoomButton-Highlight'] = true
    hideTextures['Interface\\Minimap\\UI-Minimap-Background'] = true 
    hideTextures[136430] = true 
    hideTextures[136467] = true 

    function zrMM:MinimapButtonResizeMove()
        local last = nil
		local first = nil
		local buttonCount = 0

        hideButtons = {}
		
		if zrMM.buttonPosChanged == true then
			Minimap.buttonFrame:ClearAllPoints()
			if (zrMM.config.buttonpos == "Top") then
				Minimap.buttonFrame:SetSize(Minimap:GetWidth(),zrMM.config.buttonsize)
				Minimap.buttonFrame:SetPoint("BOTTOMLEFT", Minimap, "TOPLEFT", 2, 5)
				
			elseif (zrMM.config.buttonpos == "Right") then
				Minimap.buttonFrame:SetSize(zrMM.config.buttonsize,Minimap:GetHeight())
				Minimap.buttonFrame:SetPoint("TOPLEFT", Minimap, "TOPRIGHT", 5, 0)
				
			elseif (zrMM.config.buttonpos == "Bottom") then
				Minimap.buttonFrame:SetSize(Minimap:GetWidth(),zrMM.config.buttonsize)
				Minimap.buttonFrame:SetPoint("TOPLEFT", Minimap, "BOTTOMLEFT", 2, -5)
				
			elseif (zrMM.config.buttonpos == "Left") then
				Minimap.buttonFrame:SetSize(zrMM.config.buttonsize,Minimap:GetHeight())
				Minimap.buttonFrame:SetPoint("TOPRIGHT", Minimap, "TOPLEFT", -5, 0)
				
			end
			zrMM.buttonPosChanged = false
		end


        for k, f in pairs(zrButtonFrame.frameTable) do
            f:SetWidth(zrMM.config.buttonsize)
            f:SetHeight(zrMM.config.buttonsize)
            f:ClearAllPoints()

            if (hideButtons[f:GetName()]) then
                f:Hide()
                f:SetAlpha(0)
            end
			if (f:IsShown()) then
				if (zrMM.config.buttonpos == "Top") then
					if (last) then
						if buttonCount == 10 then
							f:SetPoint("BOTTOMLEFT", first, "TOPLEFT", 0, zrMM.border*3)
							buttonCount = 0
						else
							f:SetPoint("BOTTOMLEFT", last, "BOTTOMRIGHT", zrMM.border*3, 0)     
						end
					else
						f:SetPoint("BOTTOMLEFT", Minimap.buttonFrame, "BOTTOMLEFT", -2, 0)
						first = f
					end
				
				elseif (zrMM.config.buttonpos == "Bottom") then
					if (last) then
						if buttonCount == 10 then
							f:SetPoint("TOPLEFT", first, "BOTTOMLEFT", 0, -zrMM.border*3)
							buttonCount = 0
						else
							f:SetPoint("TOPLEFT", last, "TOPRIGHT", zrMM.border*3, 0)     
						end
					else
						f:SetPoint("TOPLEFT", Minimap.buttonFrame, "TOPLEFT", -2, 0)
						first = f
					end
				
				elseif (zrMM.config.buttonpos == "Right") then
					if (last) then
						if buttonCount == 10 then
							f:SetPoint("TOPLEFT", first, "TOPRIGHT", zrMM.border*3, 0)
							buttonCount = 0
						else
							f:SetPoint("TOPLEFT", last, "BOTTOMLEFT", 0, -zrMM.border*3)  
						end
					else
						f:SetPoint("TOPLEFT", Minimap.buttonFrame, "TOPLEFT", 0, 0)
						first = f
					end
				
				elseif (zrMM.config.buttonpos == "Left") then
					if (last) then
						if buttonCount == 10 then
							f:SetPoint("TOPRIGHT", first, "TOPLEFT", -zrMM.border*3, 0)
							buttonCount = 0
						else
							f:SetPoint("TOPRIGHT", last, "BOTTOMRIGHT", 0, -zrMM.border*3)        
						end
					else
						f:SetPoint("TOPRIGHT", Minimap.buttonFrame, "TOPRIGHT", 0, 0)
						first = f
					end
				end
				last = f
				buttonCount = buttonCount + 1
			end
        end
    end

    local function SkinButton(f)
        if (f.skinned) then return end

        f:SetScale(1)
        f:SetFrameStrata("MEDIUM")

        -- Skin textures
        local r = {f:GetRegions()}
        for o = 1, #r do
            if (r[o].GetTexture and r[o]:GetTexture()) then
                local tex = r[o]:GetTexture()
                r[o]:SetAllPoints(f)
                if (hideTextures[tex]) then
                    r[o]:Hide()
                elseif (not strfind(tex,"WHITE8x8")) then
                    local coord = table.concat({r[o]:GetTexCoord()})
                    if (coord == "00011011") then
                        r[o]:SetTexCoord(0.3, 0.7, 0.3, 0.7)
                        if (n == "DugisOnOffButton") then
                            r[o]:SetTexCoord(0.25, 0.75, 0.2, 0.7)
                        end
                    end
                end
            end
        end
        
        -- Create background
        zrMM:set_backdrop(f)
        f.skinned = true
    end

    
    local function MoveButtons()
        
        
        local c = {Minimap.buttonFrame:GetChildren()}
        local d = {Minimap:GetChildren()}

        if (#d ~= numChildren) then
            numChildren = #d
            zrButtonFrame.frameTable = {}

            for k, v in pairs(d) do table.insert(c,v) end
            local last = nil
            for i = 1, #c do
                local f = c[i]
                local n = f:GetName() or i;
                f.buttonindex = i

                if (f:IsShown() and not ignoreFrames[n] and (
                        (manualTarget[n])
                        or
                        (f:GetName() and (strfind(n, "LibDB") or strfind(n, "Button") or strfind(n, "Btn")))
                    )
                        
                ) then
                    SkinButton(f)
					
                    zrButtonFrame.frameTable[n] = f
					
                end
            end
        else
            for t, v in pairs(manualTarget) do
				local f = _G[t]
                if (f) then
                    local n = f:GetName() or i;
					
                    if (f:IsShown()) then
                        SkinButton(f)
						if not zrButtonFrame.frameTable[n] then
                            zrButtonFrame.frameTable[n] = f
						end
                    else
					    if zrButtonFrame.frameTable[n] then
                            zrButtonFrame.frameTable[n] = f
						end
                    end
                end

            end
        end

        zrMM:MinimapButtonResizeMove()
    end

    local total = 0
    Minimap.buttonFrame:SetScript("OnEvent",function(self, event)
		if (event == "MAIL_CLOSED") then
			MoveButtons()
		end
	end)
    Minimap.buttonFrame:SetScript("OnUpdate", function(self, elapsed)
        total = total + elapsed
        if (total > 1) then
            total = 0
                MoveButtons()
        end
    end)
end
