local addonName, zrMM = ...

-- Default Config
zrMM.defconfig = {
			["lock"] = false,
			["size"] = 240,
			["mm"] = {
			    ["point"]="TOPRIGHT",
				["relativePoint"]="TOPRIGHT",
				["xOfs"]=0,
				["yOfs"]=0
				},
			["buttonsize"] = 18,
			["buttonpos"] = "Top",
			["clock"] = true,
			["unlockdurability"] = false,
			["zonetext"] = false,
			["durability"] = {
				["point"]="TOPRIGHT",
				["relativeto"]="Minimap",
				["relativePoint"]="BOTTOMRIGHT",
				["xOfs"]=0,
				["yOfs"]=0
			},
			["loot1"] = {
				["point"]="TOP",
				["relativeto"]="UIParent",
				["relativePoint"]="TOP",
				["xOfs"]=0,
				["yOfs"]=-135
			},
			["loot2"] = {
				["point"]="TOP",
				["relativeto"]="UIParent",
				["relativePoint"]="TOP",
				["xOfs"]=0,
				["yOfs"]=-225
			},
			["loot3"] = {
				["point"]="TOP",
				["relativeto"]="UIParent",
				["relativePoint"]="TOP",
				["xOfs"]=0,
				["yOfs"]=-310
			},
			["loot4"] = {
				["point"]="TOP",
				["relativeto"]="UIParent",
				["relativePoint"]="TOP",
				["xOfs"]=0,
				["yOfs"]=-395
			},
			
}
zrMM.buttonPosChanged = true

local function bplist()
	local bplist = {["Top"] = false,["Bottom"] = false,["Left"] = false,["Right"] = false}
	bplist[zrMM.config.buttonpos] = true
	return bplist
end



--window element creation functions
local function createCheckbox(name, label, var)
	local frame = CreateFrame("CheckButton", name, zrSettings, "ChatConfigCheckButtonTemplate")
	frame.text = getglobal(frame:GetName() .. 'Text'):SetText(label);
	frame:SetHitRectInsets(0,-100,0,0)
	frame:SetChecked(zrMM.config[var])
	frame:SetScript("OnClick", function()
		zrMM.config[var] = frame:GetChecked()
		if frame:GetName() == "zrSettings_Clock" then
			TimeManagerClockTicker:SetShown(zrMM.config.clock)
		elseif frame:GetName() == "zrSettings_Zone" then
			zrMM:ZoneTextVisibility()
		end
	end
	)
	
	return frame
end


local function createButton(name, label, frame)
	local button = CreateFrame("Button", name, zrSettings, "UIPanelButtonTemplate")
	button:SetText(label)
	button.targetframe = frame
	button:SetWidth(button.Text:GetWidth()+20)
	button:SetScript('OnClick', function()
		if not button.targetframe.text then
			for i,f in pairs(button.targetframe) do
				if f:IsShown() then
					f:Hide()
				else
					f:Show()
				end
			end
		elseif button.targetframe:IsShown() then
			button.targetframe:Hide()
		else
			button.targetframe:Show()
		end
	end)
	return button
end


local function createDropdown(name, label, list)
	local frame = CreateFrame("Button", name, zrSettings, "UIDropDownMenuTemplate")
	
	UIDropDownMenu_Initialize(frame, function()
		--create dropdown options
		for option, value in pairs(list) do 
			local info = UIDropDownMenu_CreateInfo()
			info.text = option
			info.arg1 = option
			info.arg2 = value
			if frame:GetName() == "zrSettings_ButtonPos" then
				info.func = function(self, arg1, arg2, checked)
					zrMM.config.buttonpos = arg1
					zrMM.buttonPosChanged = true
					zrMM:MinimapButtonResizeMove()
					UIDropDownMenu_SetSelectedName(frame,arg1,arg2)
				end
			end
			UIDropDownMenu_AddButton(info)
		end
		-- set selected option
		UIDropDownMenu_SetSelectedName(frame, zrMM.config.buttonpos, true) 
		
	end)
	frame.label = frame:CreateFontString(name .. "Label","OVERLAY")
	frame.label:SetFontObject("GameFontHighlight")
	frame.label:SetText(label)
	frame.label:SetPoint("LEFT", 0,25)
	
	
	return frame
end




local function createSlider(name, label, var, minval, maxval)
	local frame = CreateFrame("Slider", name, zrSettings, "OptionsSliderTemplate")
	frame:SetWidth(400)
	frame:SetHeight(20)
	frame:SetOrientation('HORIZONTAL')
	frame:SetMinMaxValues(minval, maxval)
	frame:SetValue(zrMM.config[var])
	frame:SetObeyStepOnDrag(true)
	frame:SetValueStep(1)
	frame:SetScript("OnValueChanged", function(self, value, input)
		zrMM.config[var] = value
		frame.tooltipText = tostring(value)
		getglobal(frame:GetName() .. 'Text'):SetText(label .. '\n' .. zrMM.config[var])
		if frame:GetName() == "zrSettings_Size" then
			Minimap.background:SetSize(zrMM.config.size,zrMM.config.size)
			Minimap:SetSize(zrMM.config.size, zrMM.config.size)
		end
	end)
	
	getglobal(frame:GetName() .. 'Low'):SetText(tostring(minval))
	getglobal(frame:GetName() .. 'High'):SetText(tostring(maxval))
	getglobal(frame:GetName() .. 'Text'):SetText(label .. '\n' .. zrMM.config[var])
	
	return frame
end


--create settings window
function zrMM:CreateSettingsWindow()
	zrSettings = CreateFrame("Frame", "zrSettingsFrame", UIParent)
	zrSettings:SetBackdrop({bgFile='Interface\\DialogFrame\\UI-DialogBox-Background', edgeFile='Interface\\DialogFrame\\UI-DialogBox-Border', edgeSize=32, insets={bottom=10,left=11,right=11,top=12}, tile=true, tileEdge=true, tileSize=32})
	zrSettings:SetWidth(500)
	zrSettings:SetHeight(500)
	zrSettings:SetPoint("CENTER", UIParent, "CENTER")
	zrSettings:Hide()
	zrSettings:SetMovable(true)
	zrSettings:SetHitRectInsets(0,0,0,280)
	
	--close button
	zrSettings.closeButton = CreateFrame("button", "zrSettingsCloseButton", zrSettings, 'UIPanelCloseButton')
	zrSettings.closeButton:SetPoint("TOPRIGHT", zrSettings, "TOPRIGHT", -5,-5)
	
	--header text
	zrSettings.header = zrSettings:CreateTexture("zrSettingsHeader", "ARTWORK")
	zrSettings.header:SetTexture('Interface\\DialogFrame\\UI-DialogBox-Header')
	zrSettings.header:SetPoint("TOP", zrSettings,0,15)
	zrSettings.header:SetWidth(300)
	
	zrSettings.title = zrSettings:CreateFontString("zrSettingsTitle","OVERLAY")
	zrSettings.title:SetPoint("CENTER", zrSettings.header,0,13)
	zrSettings.title:SetFontObject("zrMM_FONT_MEDIUM")
	zrSettings.title:SetText("zrMinimap Settings")
	zrSettings:SetScript('OnMouseDown', function(self, button)
	    if (button == "LeftButton") then
			zrSettings:StartMoving()
		end
    end)
	
	zrSettings:SetScript('OnMouseUp', function(self, button)
		if (button == "LeftButton") then
			zrSettings:StopMovingOrSizing()
		end
	end)
	--add minimap settings
	zrSettings.lock = createCheckbox("zrSettings_Lock", "Lock", "lock")
	zrSettings.clock = createCheckbox("zrSettings_Clock", "Show Clock", "clock")
	zrSettings.zone = createCheckbox("zrSettings_Zone", "Always Show Zone", "zonetext")
	zrSettings.size = createSlider("zrSettings_Size", "Minimap Size", "size", 50, 500)
	zrSettings.buttonsize = createSlider("zrSettings_ButtonSize", "Minimap Button Size", "buttonsize", 10, 100)
	zrSettings.buttonpos = createDropdown("zrSettings_ButtonPos", "Minimap Buttons Anchor:", bplist())
	
	
	-- add Tweaks Settings
	zrSettings.tweakUnlockText = zrSettings:CreateFontString("zrSettingsTweaksText","OVERLAY")
	zrSettings.tweakUnlockText:SetPoint("TOPLEFT", 50,-300)
	zrSettings.tweakUnlockText:SetFontObject("GameFontHighlight")
	zrSettings.tweakUnlockText:SetText("Unlock:")
	zrSettings.unlockDurability = createButton("zrSettings_UnlockDurability", "Durability Figure", zrMM.Durability)
	zrSettings.unlockGroupLoot = createButton("zrSettings_UnlockGroupLoot", "Loot Rolls", {zrMM.Loot1,zrMM.Loot2,zrMM.Loot3,zrMM.Loot4})
	
	--Minimap settings element setup
	zrSettings.lock:SetPoint("TOPLEFT", 50, -50)
	zrSettings.clock:SetPoint("TOPLEFT", 50, -70)
	zrSettings.zone:SetPoint("TOPLEFT", 200, -70)
	zrSettings.size:SetPoint("TOPLEFT", 50, -130)
	zrSettings.buttonsize:SetPoint("TOPLEFT", 50, -185)
	zrSettings.buttonpos:SetPoint("TOPLEFT", 50, -250)
	
	-- Tweaks settings element setup
	zrSettings.unlockDurability:SetPoint("TOPLEFT", 50,-320)
	zrSettings.unlockGroupLoot:SetPoint("TOPLEFT", 50,-345)
end




-- apply settings
function zrMM:ApplyConfig()
	--minimap settings
	if not IsAddOnLoaded("Blizzard_TimeManager") then
		LoadAddOn('Blizzard_TimeManager')
	end
	if (zrMM.config.clock) then
		TimeManagerClockButton:SetAlpha(1)
		TimeManagerClockButton:Show()
	else
		TimeManagerClockButton:SetAlpha(0)
		TimeManagerClockButton:Hide()
	end
	Minimap:SetSize(zrMM.config.size, zrMM.config.size)
	Minimap:SetHitRectInsets(0, 0, 0, 0)
	Minimap:SetClampRectInsets(0, 0, 0, 0)
	
	

	
end






--border functions
function zrMM:get_border(frame)
		local screenheight = select(2, GetPhysicalScreenSize())
		local scale = 768 / screenheight
		local frame_scale = frame:GetEffectiveScale()
		local pixel = scale / frame_scale
		local border = pixel * 2

		return border
	end

function zrMM:border_gen(parent)
		local frame = parent:CreateTexture(nil, "BACKGROUND", nil, -8)
		frame:SetTexture(zrMM.media.flat)
		frame:SetVertexColor(.03, .04, .05, 1)
		frame.protected = true

		return frame
	end

function zrMM:set_backdrop(frame, alpha)
	local border = zrMM:get_border(frame)
	alpha = alpha or 0.9
	local r, g, b, a = .08, .09, .11, 0.8

	if (not frame.zr_background) then
		frame.zr_background = frame:CreateTexture(nil, "BACKGROUND", nil, -7)
		frame.zr_background:SetTexture(zrMM.media.flat)
		frame.zr_background:SetAllPoints()
		frame.zr_background:SetVertexColor(r, g, b, alpha)
		frame.zr_background.protected = true

		frame.t = zrMM:border_gen(frame)
		frame.t:SetPoint("BOTTOMLEFT", frame.zr_background, "TOPLEFT", -border, 0)
		frame.t:SetPoint("BOTTOMRIGHT", frame.zr_background, "TOPRIGHT", border, 0)

		frame.l = zrMM:border_gen(frame)
		frame.l:SetPoint("TOPRIGHT", frame.zr_background, "TOPLEFT", 0, border)
		frame.l:SetPoint("BOTTOMRIGHT", frame.zr_background, "BOTTOMLEFT", 0, -border)

		frame.r = zrMM:border_gen(frame)
		frame.r:SetPoint("TOPLEFT", frame.zr_background, "TOPRIGHT", 0, border)
		frame.r:SetPoint("BOTTOMLEFT", frame.zr_background, "BOTTOMRIGHT", 0, -border)

		frame.b = zrMM:border_gen(frame)
		frame.b:SetPoint("TOPLEFT", frame.zr_background, "BOTTOMLEFT", -border, 0)
		frame.b:SetPoint("TOPRIGHT", frame.zr_background, "BOTTOMRIGHT", border, 0)

		frame.border = frame:CreateTexture(nil, "BACKGROUND")
		frame.border:Hide()
		frame.border.SetVertexColor = function(self, r, g, b, a)
			frame.t:SetVertexColor(r, g, b, a)
			frame.b:SetVertexColor(r, g, b, a)
			frame.l:SetVertexColor(r, g, b, a)
			frame.r:SetVertexColor(r, g, b, a)
		end
	end

	frame.t:SetHeight(border)
	frame.b:SetHeight(border)
	frame.l:SetWidth(border)
	frame.r:SetWidth(border)
end

